@@ -1173,7 +1173,7 @@ parse_response_code( imap_store_t *ctx, imap_cmd_t *cmd, char *s )
 		if (!(arg = next_arg( &s )) ||
 		    (ctx->uidnext = strtoul( arg, &earg, 10 ), *earg))
 		{
-			error( "IMAP error: malformed NEXTUID status\n" );
+			error( "IMAP error: malformed UIDNEXT status\n" );
 			return RESP_CANCEL;
 		}
 	} else if (!strcmp( "CAPABILITY", arg )) {
@@ -1197,17 +1197,16 @@ parse_response_code( imap_store_t *ctx, imap_cmd_t *cmd, char *s )
 	return RESP_OK;
 }
 
+static int parse_list_rsp_p1( imap_store_t *, list_t *, char * );
 static int parse_list_rsp_p2( imap_store_t *, list_t *, char * );
 
 static int
 parse_list_rsp( imap_store_t *ctx, list_t *list, char *cmd )
 {
-	char *arg;
 	list_t *lp;
 
 	if (!is_list( list )) {
 		free_list( list );
-	  bad_list:
 		error( "IMAP error: malformed LIST response\n" );
 		return LIST_BAD;
 	}
@@ -1217,15 +1216,36 @@ parse_list_rsp( imap_store_t *ctx, list_t *list, char *cmd )
 			return LIST_OK;
 		}
 	free_list( list );
-	if (!(arg = next_arg( &cmd )))
-		goto bad_list;
-	if (!ctx->delimiter[0])
-		ctx->delimiter[0] = arg[0];
-	return parse_list( ctx, cmd, parse_list_rsp_p2 );
+	return parse_list( ctx, cmd, parse_list_rsp_p1 );
 }
 
+static int
+parse_list_rsp_p1( imap_store_t *ctx, list_t *list, char *cmd ATTR_UNUSED )
+{
+	if (!is_opt_atom( list )) {
+		error( "IMAP error: malformed LIST response\n" );
+		free_list( list );
+		return LIST_BAD;
+	}
+	if (!ctx->delimiter[0] && is_atom( list ))
+		ctx->delimiter[0] = list->val[0];
+	return parse_list( ctx, cmd, parse_list_rsp_p2 );
+}
+
+// Use this to check whether a full path refers to the actual IMAP INBOX.
 static int
 is_inbox( imap_store_t *ctx, const char *arg, int argl )
+{
+	if (!starts_with_upper( arg, argl, "INBOX", 5 ))
+		return 0;
+	if (arg[5] && arg[5] != ctx->delimiter[0])
+		return 0;
+	return 1;
+}
+
+// Use this to check whether a path fragment collides with the canonical INBOX.
+static int
+is_INBOX( imap_store_t *ctx, const char *arg, int argl )
 {
 	if (!starts_with( arg, argl, "INBOX", 5 ))
 		return 0;
@@ -1248,16 +1268,22 @@ parse_list_rsp_p2( imap_store_t *ctx, list_t *list, char *cmd ATTR_UNUSED )
 	}
 	arg = list->val;
 	argl = list->len;
-	if ((l = strlen( ctx->prefix ))) {
-		if (starts_with( arg, argl, ctx->prefix, l )) {
-			arg += l;
-			argl -= l;
-			if (is_inbox( ctx, arg, argl )) {
-				if (!arg[5])
-					warn( "IMAP warning: ignoring INBOX in %s\n", ctx->prefix );
-				goto skip;
-			}
-		} else if (!is_inbox( ctx, arg, argl )) {
+	if (is_inbox( ctx, arg, argl )) {
+		// The server might be weird and have a non-uppercase INBOX. It
+		// may legitimately do so, but we need the canonical spelling.
+		memcpy( arg, "INBOX", 5 );
+	} else if ((l = strlen( ctx->prefix ))) {
+		if (!starts_with( arg, argl, ctx->prefix, l ))
+			goto skip;
+		arg += l;
+		argl -= l;
+		// A folder named "INBOX" would be indistinguishable from the
+		// actual INBOX after prefix stripping, so drop it. This applies
+		// only to the fully uppercased spelling, as our canonical box
+		// names are case-sensitive (unlike IMAP's INBOX).
+		if (is_INBOX( ctx, arg, argl )) {
+			if (!arg[5])  // No need to complain about subfolders as well.
+				warn( "IMAP warning: ignoring INBOX in %s\n", ctx->prefix );
 			goto skip;
 		}
 	}
@@ -1867,7 +1893,7 @@ ensure_password( imap_server_conf_t *srvc )
 	if (cmd) {
 		FILE *fp;
 		int ret;
-		char buffer[80];
+		char buffer[2048];  // Hopefully more than enough room for XOAUTH2, etc. tokens
 
 		if (*cmd == '+') {
 			flushn();
@@ -2396,9 +2422,15 @@ imap_open_box_p3( imap_store_t *ctx, imap_cmd_t *gcmd, int response )
 {
 	imap_cmd_open_box_t *cmdp = (imap_cmd_open_box_t *)gcmd;
 
-	// This will happen if the box is empty.
-	if (!ctx->uidnext)
+	if (!ctx->uidnext) {
+		if (ctx->total_msgs) {
+			error( "IMAP error: querying server for highest UID failed\n" );
+			imap_open_box_p4( ctx, cmdp, RESP_NO );
+			return;
+		}
+		// This is ok, the box is simply empty.
 		ctx->uidnext = 1;
+	}
 
 	imap_open_box_p4( ctx, cmdp, response );
 }
@@ -2975,10 +3007,16 @@ imap_find_new_msgs_p3( imap_store_t *ctx, imap_cmd_t *gcmd, int response )
 	imap_cmd_find_new_t *cmdp = (imap_cmd_find_new_t *)gcmd;
 	imap_cmd_find_new_t *cmd;
 
-	if (response != RESP_OK || ctx->uidnext <= cmdp->uid) {
+	if (response != RESP_OK) {
 		imap_find_new_msgs_p4( ctx, gcmd, response );
 		return;
 	}
+	if (!ctx->uidnext) {
+		// We are assuming that the new messages were not in fact instantly deleted.
+		error( "IMAP error: re-querying server for highest UID failed\n" );
+		imap_find_new_msgs_p4( ctx, gcmd, RESP_NO );
+		return;
+	}
 	INIT_IMAP_CMD(imap_cmd_find_new_t, cmd, cmdp->callback, cmdp->callback_aux)
 	cmd->out_msgs = cmdp->out_msgs;
 	imap_exec( (imap_store_t *)ctx, &cmd->gen, imap_find_new_msgs_p4,
