@@ -534,7 +534,10 @@ map_name( const char *arg, char **result, int reserve, const char *in, const cha
 static int
 compare_uints( const void *l, const void *r )
 {
-	return *(uint *)l - *(uint *)r;
+	uint li = *(uint *)l, ri = *(uint *)r;
+	if (li != ri)  // Can't subtract, the result might not fit into signed int.
+		return li > ri ? 1 : -1;
+	return 0;
 }
 
 void
